clear all; close all; clc
% Example code for 
% 
%                 Jump Regressions
% 
% by
%
% Jia Li, Department of Economics, Duke University, Durham, NC 27708; e-mail: jl410@duke.edu
% Viktor Todorov, Department of Finance, Kellogg School of Management, Northwestern University, Evanston, IL 60208; e-mail: v-todorov@northwestern.edu.
% George Tauchen, Department of Economics, Duke University, Durham, NC 27708; e-mail: george.tauchen@duke.edu
% -----------------------------------------------------------------------------------------
%
% Code runs over full sample, six years, and year-by-year as well.
% Input the data
load('ExampleData.mat');
ExampleData=Data;

% initialize some stuff
rand('seed',19330407); %April 7, 1933
randn('seed',18650409); %April 9, 1865

% bucket for 12 outcome statistics, six years and full sample
D = zeros(7,12);

% more initialization
X = ExampleData(:,1);
Y = ExampleData(:,2);
freq = 38; kn = 19; T = floor(size(X,1)/freq);
alpha_jump = 4.0; alpha_cont = 3;

% Comppute cv, bv, and tod (time of daay for market)
bv_X = zeros(T,1); rv_X = zeros(T,1);
bv_Y = zeros(T,1); rv_Y = zeros(T,1);
for t=1:T
    bv_X(t) = (freq/(freq-1))*(pi/2)*sum(abs(X((t-1)*freq+2:t*freq).*X((t-1)*freq+1:t*freq-1)));
    rv_X(t) = sum(X((t-1)*freq+1:t*freq).^2);
    bv_Y(t) = (freq/(freq-1))*(pi/2)*sum(abs(Y((t-1)*freq+2:t*freq).*Y((t-1)*freq+1:t*freq-1)));
    rv_Y(t) = sum(Y((t-1)*freq+1:t*freq).^2);
end
tod = zeros(freq,1);
for t=1:T
    for j=1:freq
        if abs(X((t-1)*freq+j))<alpha_cont*(1/freq)^0.49*sqrt(bv_X(t))
            tod(j) = tod(j)+X((t-1)*freq+j)^2;
        end
    end
end
tod = tod*freq/sum(tod);

% Find market jumps
JI  = zeros(T*freq,1);
for t=1:T
    for j=1:freq
        if abs(X((t-1)*freq+j,1))>alpha_jump*(1/freq^0.49)*sqrt(bv_X(t,1)*tod(j)) && (t-1)*freq+j>kn && (t-1)*freq+j<T*freq-kn
            JI((t-1)*freq+j,1) = 1;
        end
    end
end
JI = (JI==1);
Xsave = X;
Ysave = Y;
JIsave = JI;
bv_Xsave = bv_X;
bv_Ysave = bv_Y;

% Begin main loop
for k=1:7
    if k<7
        range = ((k-1)*freq*250+1):1:k*freq*250;
    else
        range = 1:1:6*freq*250;
    end
        % Subset variables to range
        X = Xsave(range);
        Y = Ysave(range);
        JI = JIsave(range);
        JI = (JI==1);
        d1 = floor(range(1)/freq)+1;
        d2 = floor(range(end)/freq);
        bv_X = bv_Xsave(d1:1:d2);
        bv_Y = bv_Ysave(d1:1:d2);
        T = floor(size(X,1)/freq);
        beta_OLS = sum(X(JI).*Y(JI))/sum(X(JI).^2);
        evar_m = zeros(T*freq,1); evar_p = zeros(T*freq,1);
        for i=kn+1:T*freq-kn
            count_m = 0; count_p = 0;
            for j=1:kn
                ind=mod(i-j,freq);
                if ind==0
                    ind=freq;
                end
                if  ~(JI(i-j)) ...
                        && (abs(Y(i-j))<alpha_cont*(1/freq)^0.49*sqrt(bv_Y(ceil((i-j)/freq))*tod(ind)))
                    evar_m(i) =evar_m(i)+(Y(i-j)-beta_OLS*X(i-j))^2; count_m = count_m+1;
                end
            end
            for j=1:kn
                ind=mod(i+j,freq);
                if ind==0
                    ind=freq;
                end
                if ~(JI(i+j)) ...
                        && (abs(Y(i+j))<alpha_cont*(1/freq)^0.49*sqrt(bv_Y(ceil((i+j)/freq))*tod(ind)))
                    evar_p(i) =evar_p(i)+(Y(i+j)-beta_OLS*X(i+j))^2; count_p = count_p+1;
                end
            end
            evar_m(i) = evar_m(i)*freq/count_m; evar_p(i) = evar_p(i)*freq/count_p;
        end
        
        % Various statistics
        Test_Val = sum(Y(JI).^2)*sum(X(JI).^2)-(sum(X(JI).*Y(JI)))^2; Test_Val = Test_Val./(sum(Y(JI).^2)*sum(X(JI).^2)); Test_Val = Test_Val*freq;
        nuis = [X(JI) Y(JI) sqrt(evar_m(JI)) sqrt(evar_p(JI))];
        beta_eff = eb(nuis); [ ci_eff, ci] = eb_ci(beta_OLS,nuis,freq,kn);
        
        % simquat returns probability value as per paper.
        out = [sum(JI) beta_eff ci_eff beta_OLS ci sqrt(1-Test_Val/freq) Test_Val/freq simquant(nuis, Test_Val)];
        D(k,:) = out;
        
        % Main loop ends here
    end
    
    % Display test statistics by year and full sample; other variables are in
    % the matrix D
    [D(:,1) D(:,end)]'
    
    % Scatter Plot
    close all
    figure()
    scatter(X(JI),Y(JI)); hold; plot(X(JI),beta_OLS*X(JI)); plot(X(JI),beta_eff*X(JI),'r');
    
